using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Xml;
using System.Net;
using System.IO;
using System.Collections;

namespace SOAPClientExample
{
    class SOAPClient
    {
        public const string username = null; //put your username here
        public const string password = null; //put your password here

        static void Main()
        {
            Debug.WriteLine("---RUNNING SOAP CLIENT---");
            CallWebService();
        }


        public static void CallWebService()
        {
            var _url = "https://infoconnect1.highwayinfo.govt.nz:443/ic/jbi/TrafficCameras2/SOAP/FeedService/";
            var _action = "";

            XmlDocument soapEnvelopeXml = CreateSoapEnvelope();
            HttpWebRequest webRequest = CreateWebRequest(_url, _action);
            InsertSoapEnvelopeIntoWebRequest(soapEnvelopeXml, webRequest);

            // begin async call to web request.
            IAsyncResult asyncResult = webRequest.BeginGetResponse(null, null);

            // suspend this thread until call is complete. You might want to
            // do something usefull here like update your UI.
            asyncResult.AsyncWaitHandle.WaitOne();

            // get the response from the completed web request.
            string soapResult;
            using (WebResponse webResponse = webRequest.EndGetResponse(asyncResult))
            {
                using (StreamReader rd = new StreamReader(webResponse.GetResponseStream()))
                {
                    soapResult = rd.ReadToEnd();
                }

                Debug.WriteLine("RECEIVED RESPONSE FROM SERVER");
                Debug.WriteLine(soapResult);

                XmlDocument responseXML = new XmlDocument();
                responseXML.LoadXml(soapResult);
                XmlNode camerasList = responseXML.FirstChild.FirstChild.FirstChild;

                Debug.WriteLine("CAMERAS");
                foreach (XmlNode xml in camerasList.ChildNodes)
                {
                    if (xml.InnerText.Contains("Wellington"))
                    {
                        Debug.WriteLine(xml.InnerText);
                    }
                }
            }
        }

        private static HttpWebRequest CreateWebRequest(string url, string action)
        {
            HttpWebRequest webRequest = (HttpWebRequest)WebRequest.Create(url);
            webRequest.Headers.Add("username", username);
            webRequest.Headers.Add("password", password);
            webRequest.ContentType = "text/xml";
            webRequest.Accept = "text/xml";
            webRequest.Method = "POST";
            return webRequest;
        }

        private static XmlDocument CreateSoapEnvelope()
        {
            XmlDocument soapEnvelopeDocument = new XmlDocument();

            soapEnvelopeDocument.LoadXml(
              @"<soapenv:Envelope xmlns:soapenv=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:cam=""https://infoconnect.highwayinfo.govt.nz/schemas/camera2"">"
            + @"   <soapenv:Header>"
            + @"      <wsse:Security xmlns:wsse=""http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"">"
            + @"         <wsse:UsernameToken >"
            + @"            <wsse:Username>" + username + @"</wsse:Username>"
            + @"            <wsse:Password Type=""http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText"">" + password + @"</wsse:Password>"
            + @"         </wsse:UsernameToken>"
            + @"      </wsse:Security>"
            + @"    </soapenv:Header>"
            + @"   <soapenv:Body>"
            + @"      <cam:getCamerasRequest/>"
            + @"   </soapenv:Body>"
            + @"</soapenv:Envelope>"
            );
            return soapEnvelopeDocument;
        }

        private static void InsertSoapEnvelopeIntoWebRequest(XmlDocument soapEnvelopeXml, HttpWebRequest webRequest)
        {
            using (Stream stream = webRequest.GetRequestStream())
            {
                soapEnvelopeXml.Save(stream);
            }
        }

    }


}
